'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var slatePluginsCommon = require('@udecode/slate-plugins-common');
var React = require('react');
require('slate-react');
var slate = require('slate');

const MARK_BOLD = 'bold';
const DEFAULTS_BOLD = {
  hotkey: 'mod+b'
};

function create$1(createState) {
  let state;
  const listeners = new Set();

  const setState = (partial, replace) => {
    const nextState = typeof partial === 'function' ? partial(state) : partial;

    if (nextState !== state) {
      const previousState = state;
      state = replace ? nextState : Object.assign({}, state, nextState);
      listeners.forEach(listener => listener(state, previousState));
    }
  };

  const getState = () => state;

  const subscribeWithSelector = (listener, selector = getState, equalityFn = Object.is) => {
    let currentSlice = selector(state);

    function listenerToAdd() {
      const nextSlice = selector(state);

      if (!equalityFn(currentSlice, nextSlice)) {
        const previousSlice = currentSlice;
        listener(currentSlice = nextSlice, previousSlice);
      }
    }

    listeners.add(listenerToAdd); // Unsubscribe

    return () => listeners.delete(listenerToAdd);
  };

  const subscribe = (listener, selector, equalityFn) => {
    if (selector || equalityFn) {
      return subscribeWithSelector(listener, selector, equalityFn);
    }

    listeners.add(listener); // Unsubscribe

    return () => listeners.delete(listener);
  };

  const destroy = () => listeners.clear();

  const api = {
    setState,
    getState,
    subscribe,
    destroy
  };
  state = createState(setState, getState, api);
  return api;
}

const useIsoLayoutEffect = typeof window === 'undefined' ? React.useEffect : React.useLayoutEffect;
function create(createState) {
  const api = typeof createState === 'function' ? create$1(createState) : createState;

  const useStore = (selector = api.getState, equalityFn = Object.is) => {
    const [, forceUpdate] = React.useReducer(c => c + 1, 0);
    const state = api.getState();
    const stateRef = React.useRef(state);
    const selectorRef = React.useRef(selector);
    const equalityFnRef = React.useRef(equalityFn);
    const erroredRef = React.useRef(false);
    const currentSliceRef = React.useRef();

    if (currentSliceRef.current === undefined) {
      currentSliceRef.current = selector(state);
    }

    let newStateSlice;
    let hasNewStateSlice = false; // The selector or equalityFn need to be called during the render phase if
    // they change. We also want legitimate errors to be visible so we re-run
    // them if they errored in the subscriber.

    if (stateRef.current !== state || selectorRef.current !== selector || equalityFnRef.current !== equalityFn || erroredRef.current) {
      // Using local variables to avoid mutations in the render phase.
      newStateSlice = selector(state);
      hasNewStateSlice = !equalityFn(currentSliceRef.current, newStateSlice);
    } // Syncing changes in useEffect.


    useIsoLayoutEffect(() => {
      if (hasNewStateSlice) {
        currentSliceRef.current = newStateSlice;
      }

      stateRef.current = state;
      selectorRef.current = selector;
      equalityFnRef.current = equalityFn;
      erroredRef.current = false;
    });
    const stateBeforeSubscriptionRef = React.useRef(state);
    useIsoLayoutEffect(() => {
      const listener = () => {
        try {
          const nextState = api.getState();
          const nextStateSlice = selectorRef.current(nextState);

          if (!equalityFnRef.current(currentSliceRef.current, nextStateSlice)) {
            stateRef.current = nextState;
            currentSliceRef.current = nextStateSlice;
            forceUpdate();
          }
        } catch (error) {
          erroredRef.current = true;
          forceUpdate();
        }
      };

      const unsubscribe = api.subscribe(listener);

      if (api.getState() !== stateBeforeSubscriptionRef.current) {
        listener(); // state has changed before subscription
      }

      return unsubscribe;
    }, []);
    return hasNewStateSlice ? newStateSlice : currentSliceRef.current;
  };

  Object.assign(useStore, api); // For backward compatibility (No TS types for this)

  useStore[Symbol.iterator] = function* () {
    console.warn('[useStore, api] = create() is deprecated and will be removed in v4');
    yield useStore;
    yield api;
  };

  return useStore;
}

function unwrapExports(x) {
  return x && x.__esModule && Object.prototype.hasOwnProperty.call(x, 'default') ? x['default'] : x;
}

function createCommonjsModule(fn, module) {
  return module = {
    exports: {}
  }, fn(module, module.exports), module.exports;
}

var vanilla = createCommonjsModule(function (module, exports) {
  Object.defineProperty(exports, '__esModule', {
    value: true
  });

  function create(createState) {
    var state;
    var listeners = new Set();

    var setState = function setState(partial, replace) {
      var nextState = typeof partial === 'function' ? partial(state) : partial;

      if (nextState !== state) {
        var previousState = state;
        state = replace ? nextState : Object.assign({}, state, nextState);
        listeners.forEach(function (listener) {
          return listener(state, previousState);
        });
      }
    };

    var getState = function getState() {
      return state;
    };

    var subscribeWithSelector = function subscribeWithSelector(listener, selector, equalityFn) {
      if (selector === void 0) {
        selector = getState;
      }

      if (equalityFn === void 0) {
        equalityFn = Object.is;
      }

      var currentSlice = selector(state);

      function listenerToAdd() {
        var nextSlice = selector(state);

        if (!equalityFn(currentSlice, nextSlice)) {
          var previousSlice = currentSlice;
          listener(currentSlice = nextSlice, previousSlice);
        }
      }

      listeners.add(listenerToAdd); // Unsubscribe

      return function () {
        return listeners.delete(listenerToAdd);
      };
    };

    var subscribe = function subscribe(listener, selector, equalityFn) {
      if (selector || equalityFn) {
        return subscribeWithSelector(listener, selector, equalityFn);
      }

      listeners.add(listener); // Unsubscribe

      return function () {
        return listeners.delete(listener);
      };
    };

    var destroy = function destroy() {
      return listeners.clear();
    };

    var api = {
      setState: setState,
      getState: getState,
      subscribe: subscribe,
      destroy: destroy
    };
    state = createState(setState, getState, api);
    return api;
  }

  exports.default = create;
});
var createVanillaStore = unwrapExports(vanilla);
/**
 * Slate plugins vanilla store.
 * @see zustand vanilla store
 */

const slatePluginsStore = createVanillaStore(() => ({}));
/**
 * Slate plugins store.
 * @see zustand store
 */

create(slatePluginsStore);

slatePluginsStore.setState;

const s = slatePluginsStore;
s.getState;
s.setState;

const getOptions = editor => {
  var _a, _b;

  return (_b = (_a = editor) === null || _a === void 0 ? void 0 : _a.options) !== null && _b !== void 0 ? _b : {};
};

const getPluginOptions = (editor, pluginKey) => {
  var _a;

  return (_a = getOptions(editor)[pluginKey]) !== null && _a !== void 0 ? _a : {};
};
slate.Element.isElement;

slate.Text.isText;

const useDeserializeBold = () => editor => {
  const options = getPluginOptions(editor, MARK_BOLD);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['STRONG']
      }, {
        style: {
          fontWeight: ['600', '700', 'bold']
        }
      }]
    }, options.deserialize))
  };
};

/**
 * Enables support for bold formatting
 */

const useBoldPlugin = () => ({
  pluginKeys: MARK_BOLD,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_BOLD),
  deserialize: useDeserializeBold(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_BOLD)
});

const MARK_CODE = 'code';
const DEFAULTS_CODE = {
  hotkey: 'mod+e'
};

const useDeserializeCode = () => editor => {
  const options = getPluginOptions(editor, MARK_CODE);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['CODE']
      }, {
        style: {
          wordWrap: 'break-word'
        }
      }]
    }, options.deserialize))
  };
};

/**
 * Enables support for code formatting
 */

const useCodePlugin = () => ({
  pluginKeys: MARK_CODE,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_CODE),
  deserialize: useDeserializeCode(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_CODE)
});

const MARK_ITALIC = 'italic';
const DEFAULTS_ITALIC = {
  hotkey: 'mod+i'
};

/**
 * Enables support for italic formatting.
 */

const useItalicPlugin = () => ({
  pluginKeys: MARK_ITALIC,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_ITALIC),
  deserialize: useDeserializeCode(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_ITALIC)
});

const MARK_UNDERLINE = 'underline';
const DEFAULTS_UNDERLINE = {
  hotkey: 'mod+u'
};

const useDeserializeUnderline = () => editor => {
  const options = getPluginOptions(editor, MARK_UNDERLINE);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['U']
      }, {
        style: {
          textDecoration: 'underline'
        }
      }]
    }, options.deserialize))
  };
};

const MARK_STRIKETHROUGH = 'strikethrough';
const DEFAULTS_STRIKETHROUGH = {
  hotkey: 'mod+shift+s'
};

/**
 * Enables support for strikethrough formatting.
 */

const useStrikethroughPlugin = () => ({
  pluginKeys: MARK_STRIKETHROUGH,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_STRIKETHROUGH),
  deserialize: useDeserializeUnderline(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_STRIKETHROUGH)
});

const MARK_SUBSCRIPT = 'subscript';
const DEFAULTS_SUBSCRIPT = {
  hotkey: 'mod+.',
  clear: MARK_SUBSCRIPT
};

/**
 * Enables support for subscript formatting.
 */

const useSubscriptPlugin = () => ({
  pluginKeys: MARK_SUBSCRIPT,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_SUBSCRIPT),
  deserialize: useDeserializeUnderline(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_SUBSCRIPT)
});

const MARK_SUPERSCRIPT = 'superscript';
const DEFAULTS_SUPERSCRIPT = {
  hotkey: 'mod+,',
  clear: MARK_SUPERSCRIPT
};

/**
 * Enables support for superscript formatting.
 */

const useSuperscriptPlugin = () => ({
  pluginKeys: MARK_SUPERSCRIPT,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_SUPERSCRIPT),
  deserialize: useDeserializeUnderline(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_SUPERSCRIPT)
});

/**
 * Enables support for underline formatting.
 */

const useUnderlinePlugin = () => ({
  pluginKeys: MARK_UNDERLINE,
  renderLeaf: slatePluginsCommon.useRenderLeaf(MARK_UNDERLINE),
  deserialize: useDeserializeUnderline(),
  onKeyDown: slatePluginsCommon.useOnKeyDownMark(MARK_UNDERLINE)
});

/**
 * Enables support for basic marks:
 * - Bold
 * - Code
 * - Italic
 * - Strikethrough
 * - Subscript
 * - Superscript
 * - Underline
 */

const useBasicMarkPlugins = () => [useBoldPlugin(), useCodePlugin(), useItalicPlugin(), useStrikethroughPlugin(), useSubscriptPlugin(), useSuperscriptPlugin(), useUnderlinePlugin()];

const useDeserializeItalic = () => editor => {
  const options = getPluginOptions(editor, MARK_ITALIC);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['EM', 'I']
      }, {
        style: {
          fontStyle: 'italic'
        }
      }]
    }, options.deserialize))
  };
};

const useDeserializeStrikethrough = () => editor => {
  const options = getPluginOptions(editor, MARK_STRIKETHROUGH);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['S', 'DEL', 'STRIKE']
      }, {
        style: {
          textDecoration: 'line-through'
        }
      }]
    }, options.deserialize))
  };
};

const useDeserializeSubscript = () => editor => {
  const options = getPluginOptions(editor, MARK_SUBSCRIPT);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['SUB']
      }, {
        style: {
          verticalAlign: 'sub'
        }
      }]
    }, options.deserialize))
  };
};

const useDeserializeSuperscript = () => editor => {
  const options = getPluginOptions(editor, MARK_SUPERSCRIPT);
  return {
    leaf: slatePluginsCommon.getLeafDeserializer(Object.assign({
      type: options.type,
      rules: [{
        nodeNames: ['SUP']
      }, {
        style: {
          verticalAlign: 'super'
        }
      }]
    }, options.deserialize))
  };
};

exports.DEFAULTS_BOLD = DEFAULTS_BOLD;
exports.DEFAULTS_CODE = DEFAULTS_CODE;
exports.DEFAULTS_ITALIC = DEFAULTS_ITALIC;
exports.DEFAULTS_STRIKETHROUGH = DEFAULTS_STRIKETHROUGH;
exports.DEFAULTS_SUBSCRIPT = DEFAULTS_SUBSCRIPT;
exports.DEFAULTS_SUPERSCRIPT = DEFAULTS_SUPERSCRIPT;
exports.DEFAULTS_UNDERLINE = DEFAULTS_UNDERLINE;
exports.MARK_BOLD = MARK_BOLD;
exports.MARK_CODE = MARK_CODE;
exports.MARK_ITALIC = MARK_ITALIC;
exports.MARK_STRIKETHROUGH = MARK_STRIKETHROUGH;
exports.MARK_SUBSCRIPT = MARK_SUBSCRIPT;
exports.MARK_SUPERSCRIPT = MARK_SUPERSCRIPT;
exports.MARK_UNDERLINE = MARK_UNDERLINE;
exports.useBasicMarkPlugins = useBasicMarkPlugins;
exports.useBoldPlugin = useBoldPlugin;
exports.useCodePlugin = useCodePlugin;
exports.useDeserializeBold = useDeserializeBold;
exports.useDeserializeCode = useDeserializeCode;
exports.useDeserializeItalic = useDeserializeItalic;
exports.useDeserializeStrikethrough = useDeserializeStrikethrough;
exports.useDeserializeSubscript = useDeserializeSubscript;
exports.useDeserializeSuperscript = useDeserializeSuperscript;
exports.useDeserializeUnderline = useDeserializeUnderline;
exports.useItalicPlugin = useItalicPlugin;
exports.useStrikethroughPlugin = useStrikethroughPlugin;
exports.useSubscriptPlugin = useSubscriptPlugin;
exports.useSuperscriptPlugin = useSuperscriptPlugin;
exports.useUnderlinePlugin = useUnderlinePlugin;
//# sourceMappingURL=index.js.map
